/**
 * Copyright 2020 Alibaba Group Holding Limited.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.maxgraph.tests.common.rpc;

import com.alibaba.maxgraph.v2.common.config.CommonConfig;
import com.alibaba.maxgraph.v2.common.config.Configs;
import com.alibaba.maxgraph.v2.common.discovery.LocalNodeProvider;
import com.alibaba.maxgraph.v2.common.discovery.MaxGraphNode;
import com.alibaba.maxgraph.v2.common.discovery.RoleType;
import com.alibaba.maxgraph.v2.common.rpc.RpcServer;
import org.junit.jupiter.api.Test;

import java.io.IOException;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class RpcServerTest {

    @Test
    void testRpcServer() throws IOException {
        int port = 1111;
        Configs configs = Configs.newBuilder()
                .put(CommonConfig.RPC_PORT.getKey(), String.valueOf(port))
                .put(CommonConfig.ROLE_NAME.getKey(), RoleType.STORE.getName())
                .put(CommonConfig.NODE_IDX.getKey(), "0")
                .build();
        LocalNodeProvider localNodeProvider = new LocalNodeProvider(configs);
        RpcServer rpcServer = new RpcServer(configs, localNodeProvider);
        rpcServer.start();
        MaxGraphNode localNode = localNodeProvider.get();
        assertEquals(port, localNode.getPort());
        assertEquals(port, rpcServer.getPort());
        rpcServer.stop();
    }
}
